#!/bin/bash

# no comment
function PrintUsage
{
	echo $ERROR
	echo "Package roller for LiveMath products"
	echo ""
	echo "Usage: packager.sh [<options>] [<product> [<language>]]"
	echo ""
	echo "The <option> is one or more of following:"
	echo "  -c, --clean         : cleans intermediate files from build"
	echo "  -r, --rpm           : build RPM distribution package obly"
	echo "  -d, --deb           : build DEB distribution package only"
	echo "  -t, --tar           : build .tar.gz. distribution package only"
	echo "  -a, --all           : build all necessary archives (DEB, RPM and .tar.gz) (default)"
 	echo "  -n, --no-build      : doesn't perform build process, just packaging"
	echo "  -s, --silent        : if no errors - skip warning and output messages"
	echo "  -?, -h, --help      : prints this help message"
	echo ""
	echo "A <product> is one of the following: all(default), TRS, Theorist, TVS, Viewer, STR, Starters"
	echo "A <language> is given as in build.sh script"
	echo "" 
    echo "Multi-options (for example: -rd) are not allowed. However, several single-options (-r -d) are allowed."
	echo ""
	exit 0
}

function FillAFile
{
	srcfile=$1
	dstfile=$2

	sed -e "s/%VERSION%/${VERSION}/g" \
		-e "s/%RELEASE%/${RELEASE}/g" \
		-e "s/%PLATFORM%/${PLATFORM}/g" \
		-e "s/%LANG%/${LANGUAGE}/g" \
		-e "s/%ARCH%/${ARCH}/g" \
		-e "s/%SIZE%/${SIZE}/g" \
		-e "s/%BUILD_ROOT%/${BUILD_ROOT}/g" \
		$srcfile > $dstfile
}

##### BEGINING
# parse command line
CLEAN="no"
ERROR=""
BUILDRPM="no"
BUILDTAR="no"
BUILDDEB="no"
DOBUILD="yes"
SILENT="no"

optarg=1
for option in $@;
do
	if [ "${option:0:1}" == "-" ]; then
		case $option in
			-c | --clean) CLEAN="yes";;
			-r | --rpm) BUILDRPM="yes";;
			-d | --deb) BUILDDEB="yes";;
			-t | --tar) BUILDTAR="yes";;
	 		-a | --all) BUILDRPM="yes" BUILDTAR="yes" BUILDDEB="yes";;
	   		-n | --no-build) DOBUILD="no";;
			-s | --silent) SILENT="yes";;
			-? | -h | --help) PrintUsage;;
			*) ERROR="wrong parameter: $option";;
		esac
		optarg=$((optarg + 1))
	fi
done

if [ "$ERROR" != "" ]; then
	PrintUsage
fi

if [ "$BUILDRPM" == "no" ] && [ "$BUILDDEB" == "no" ] && [ "$BUILDTAR" == "no" ]; then
# i.e. - we aren't specified anything so - default it
	BUILDRPM="yes"
	BUILDDEB="yes"
	BUILDTAR="yes"
fi

# include defaults to get binary root from there
source ./default.sh
mkdir -p $BINROOT/packages > /dev/null 2>/dev/null

pushd . > /dev/null
cd $BINROOT
BINROOT=`pwd -P`
popd > /dev/null

echo "Packages will live in '${BINROOT}/packages'"

# now determine the product - similar to product.sh, but simpler
PRODNAME=${!optarg}
optarg=$((optarg + 1))

if [ "$PRODNAME" = "" ] || [ "$PRODNAME" = "all" ]; then
	PRODNAME=("livemath-maker" "livemath-viewer" "livemath-starters")
	PRODPREFIX='TRS TVS STR'
	BUILDPREFIX='TRS THS TVS'
elif [ "$PRODNAME" = "Theorist" ] || [ "$PRODNAME" = "TRS" ]; then
	PRODNAME=("livemath-maker")
	PRODPREFIX='TRS'
	BUILDPREFIX='TRS'
elif [ "$PRODNAME" = "Viewer" ] || [ "$PRODNAME" = "TVS" ]; then
	PRODNAME=("livemath-viewer")
	PRODPREFIX='TVS'
	BUILDPREFIX='THS TVS'
elif [ "$PRODNAME" = "Starters" ] || [ "$PRODNAME" = "STR" ]; then
	PRODNAME=("livemath-starters")
	PRODPREFIX='STR'
	BUILDPREFIX=
else
	ERROR="wrong product specified: ${PRODNAME}"
	PrintUsage
fi

LANGUAGE=${!optarg}
if [ "${LANGUAGE}" == "" ]; then
	LANGUAGE=all
fi

# first perform building !!!
if [ "$DOBUILD" == "yes" ]; then
	echo "Go for BUILDING: "
	for build in $BUILDPREFIX
	do	
		if [ "$CLEAN" == "yes" ]; then
			### if necessary make cleaning
			echo "Cleaning $build... "
			./clean.sh $build
			echo "Done."
		fi
		echo "Starting build.sh Release $build $LANGUAGE..."
		./build.sh Release $build $LANGUAGE
	done
	echo "Done."
fi

PLATFORM=`cat $BINROOT/platform.info`
ARCH=`sed -r "s/.*-(.*).*/\1/" $BINROOT/platform.info`

if [ "${ARCH}" == "i86" ]; then
	ARCH="i386"
fi

prodidx=0
for product in $PRODPREFIX
do
	product_name=${PRODNAME[$prodidx]}
	packdir=$BINROOT/packages/$product_name-packdir
	mkdir  $packdir > /dev/null 2>/dev/null
	echo "Preparing contents in $packdir... "
	./filesprep.${product}.sh $BINROOT $SRCROOT $packdir
	SIZE=`du -s $packdir | sed -r "s/([0-9]+).*/\1/"`
	echo "Done. Size of contents is ${SIZE} KB"

	VERSION=`cat $BINROOT/$product/version.info`
	RELEASE=`cat $BINROOT/$product/buildid.info`
	
	echo "Working on $product_name, with version=${VERSION}-${RELEASE} on platform-${ARCH}... "

	# now all contents are there - we won't make too many changes after that
	if [ "$BUILDDEB" == "yes" ]; then
		debname=${product_name}_${VERSION}-${RELEASE}_${ARCH}.deb
		echo "Rolling $debname..."

		cp -rf $SRCROOT/BuildUx/deb.$product $packdir/DEBIAN
		chmod +x $packdir/DEBIAN/postinst $packdir/DEBIAN/prerm
		rm $packdir/DEBIAN/control # it'll be filled later
		rm -rf $packdir/DEBIAN/CVS

		# now prepare the real control file
		FillAFile $SRCROOT/BuildUx/deb.$product/control $packdir/DEBIAN/control
		pushd $packdir > /dev/null
		cd ..
		dpkg-deb -b $product_name-packdir $debname
		
		# now we should clean DEBAIN-specific folder, which is .... DEBIAN
		rm -rf $packdir/DEBIAN > /dev/null
		popd > /dev/null
		echo "Done DEB."
	fi

	if [ "$BUILDRPM" == "yes" ]; then
		rpmname=${product_name}_${VERSION}-${RELEASE}_${ARCH}.rpm
		echo "Rolling $rpmname... "

		#prepare the spec file
		specfile=$packdir/../$product_name.spec
		FillAFile $SRCROOT/BuildUx/rpmspec.$product $specfile 

		# put this in RPM macros file - later to be removed
		cp ~/.rpmmacros /tmp/__rpm_macrosfile
	 	echo "%_topdir $packdir" >> ~/.rpmmacros

		pushd $packdir > /dev/null
		mkdir -p BUILD > /dev/null 2>/dev/null
		mkdir -p RPMS/${ARCH} > /dev/null 2>/dev/null
		cd ..
		
		rpmbuild -bb --target ${ARCH} --buildroot $packdir $specfile
		mv -f $packdir/RPMS/${ARCH}/* . > /dev/null
		popd > /dev/null
		
		rm -rf $packdir/BUILD > /dev/null
		rm -rf $packdir/RPMS > /dev/null
		rm -rf $packdir/var > /dev/null
		rm -f $specfile > /dev/null

		# now remove this supid conf line in .rpmmacros file
		mv -f /tmp/__rpm_macrosfile ~/.rpmmacros
		echo "Done RPM."
	fi

	# this should be last, since it changes the content structure!!!
	if [ "$BUILDTAR" == "yes" ]; then
		tarname=${product_name}_${VERSION}-${RELEASE}_${ARCH}.tar
		echo "Rolling $tarname... "
		# we'll alter the structure a bit...

		# first put some files from SRCROOT
		cp $SRCROOT/BuildUx/shared-content/install.sh $packdir > /dev/null
		chmod +x $packdir/install.sh > /dev/null
		cp $SRCROOT/BuildUx/shared-content/uninstall.sh $packdir > /dev/null
		chmod +x $packdir/uninstall.sh > /dev/null

		pushd $packdir > /dev/null
		cp -r usr/lib/$product_name/* . > /dev/null 2>/dev/null
		cp -r usr/share . > /dev/null > /dev/null 2>/dev/null
		rm -rf usr > /dev/null
		
		# modify a bit more for starters lib
		rm -f Starters
		ln -sf share/livemath/starters Starters
		
		# ready to archive
		tar -cf $tarname *
		gzip -f $tarname
		mv ${tarname}.gz .. > /dev/null
		popd > /dev/null
		echo "Done TAR.GZ."
	fi

	rm -rf $packdir > /dev/null
	prodidx=$(($prodidx + 1))
done

